package com.crm.manager.dao.mapper;

import com.crm.rely.backend.core.entity.control.panel.statistics.ControlPanelStatisticsEntity;
import com.crm.rely.backend.core.pojo.view.*;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface ControlPanelStatisticsMapper {
    ControlPanelStatisticsView searchByDate(ControlPanelStatisticsEntity entity);

    List<ControlPanelStatisticsView> searchListByDate(ControlPanelStatisticsEntity entity);

    ControlPanelCustomNumberView searchCustomNumber();

    ControlPanelManagerApplyNumberView searchManagerApplyNumber();

    ControlPanelFinanceBalancePendingNUmberView searchFinanceBalancePendingNumber();

    ControlPanelManagerNumberView searchManagerByData(ControlPanelStatisticsEntity entity);

}
